# lpc4322 HIC

Based on LPC4322JET100E chip ([Data Sheet](https://www.nxp.com/docs/en/data-sheet/LPC435X_3X_2X_1X.pdf)):
- Cortex-M4 204 Mhz (currently run at 120 MHz)
- 512 KB Flash
- 104 KB RAM
- High-speed USB 2.0 device controller: 6 bi-directional endpoints including EP0 (*)
- TFBGA100 packaging

(*) : "Supports six logical endpoints including one control endpoint for a total of 12 physical
endpoints." (source: [UM10503 :lock:](https://www.nxp.com/webapp/Download?colCode=UM10503))

## Memory Map

| Region   |  Size  | Start       | End         |
|----------|--------|-------------|-------------|
| Flash    | 200 KB | 0x1A00_0000 | 0x1A03_2000 |
| SRAM     |  32 KB | 0x1000_0000 | 0x1000_8000 |

Bootloader size is 64 KB

## DAPLink default pin assignment

| Signal      | I/O | Symbol  | Pin |
|-------------|:---:|---------|:---:|
| SWD / JTAG  |
| SWCLK / TCK |  O  | P1_17   | H10 |
| SWDIO / TMS | I/O | P1_6    |  K4 |
| SWO / TDO   |  I  | P1_14   |  J8 |
| TDI         |  O  | P0_18   | J10 |
| nRESET      |  O  | P2_5    | D10 |
| GNDDetect   |  I  | P2_13   | A10 |
| UART        |
| UART TX     |  O  | P2_0    | G10 |
| UART RX     |  I  | P2_1    |  G7 |
| Button      |
| NF-RST But. |  I  | P2_11   |  A9 |
| LEDs        |
| LED         |  O  | P1_1    |  K2 |

## Installation using ROM USB DFU support

The LPC4322 has USB DFU support in ROM which can be leveraged with
[LPCScrypt](https://www.nxp.com/design/microcontrollers-developer-resources/lpcscrypt-v2-1-2:LPCSCRYPT)
to load DAPLink bootloader.

| Board            | Label    | Jumper |
|------------------|----------|:------:|
| LPCXpresso55S69  | DFU      |  J4    |
| LPCXpresso55S28  | DFU      |  J4    |
| LPCXpresso55S16  | Link2    |  JP6   |
| LPCXpresso55S06  | DFU      |  JP6   |
| LPCXpresso54628  | DFU Link |  JP5   |
| LPCXpresso54618  | DFU Link |  JP5   |
| LPCXpresso54608  | DFU Link |  JP5   |
| LPCXpresso54114  | DFU Link |  JP5   |
| LPCXpresso54102  | DFU Link |  JP5   |
| LPCXpresso54S018 | DFU Link |  JP5   |
| LPCXpresso54018  | DFU Link |  JP5   |
| LPCXpresso43S67  | DFU Link |  JP6   |
| LPCXpresso43S37  | DFU Link |  JP6   |
| LPCXpresso18S37  | DFU Link |  JP6   |
| LPCXpresso4367   | DFU Link |  JP6   |
| LPCXpresso4337   | DFU Link |  JP6   |
| LPCXpresso1125   | DFU Link |  JP5   |

Install jumper according to table above and power cycle to force into
DFU boot mode. Adapt the following command to program
`lpc4322_bl_crc.bin`.

```
$ ./scripts/boot_lpcscrypt
Looking for DFU devices with VID 1fc9 PID 000c ...
dfu-util -d 1fc9:000c -c 1 -i 0 -t 2048 -R  -D ./scripts/../bin/LPCScrypt_240.bin.hdr
Booted LPCScrypt target (1fc9:000c) with ./scripts/../bin/LPCScrypt_240.bin.hdr
$ ./bin/lpcscrypt querypart
partID = 0xa00bcb3c 80
decode = LPC4322: BankA 512KB
Core Clock = 180000000
$ ./bin/lpcscrypt program ${PATH_TO_BINARY}/lpc4322_bl_crc.bin BANKA
..
Programmed 57344 bytes to 0x1a000000 in 0.532s (105.297KB/sec)
```

Remove the jumper and power cycle. The `MAINTENANCE` drive should
appear and you can copy the interface firmware into it. If there is no
specific interface firmware support for the board you can use the
generic `lpc4322_if_crc.bin` interface firmware which does not offer
drag and drop flash functionality.
